@@ -186,25 +186,28 @@ function (file = "./Rplots.tex", width = 7, height = 7,
   footer = getOption("tikzFooter")
 ){
 
-  tryCatch({
-    # Ok, this sucks. We copied the function signature of pdf() and got `file`
-    # as an argument to our function. We should have copied png() and used
-    # `filename`.
+  if ( !console ) {
+    # Console output does not use a file.
+    tryCatch({
+      # Ok, this sucks. We copied the function signature of pdf() and got `file`
+      # as an argument to our function. We should have copied png() and used
+      # `filename`.
 
-    # file_path_as_absolute can give us the absolute path to the output
-    # file---but it has to exist first. So, we use file() to "touch" the
-    # path.
-    touch_file <- suppressWarnings(file(file, 'w'))
-    close(touch_file)
+      # file_path_as_absolute can give us the absolute path to the output
+      # file---but it has to exist first. So, we use file() to "touch" the
+      # path.
+      touch_file <- suppressWarnings(file(file, 'w'))
+      close(touch_file)
 
-    file <- tools::file_path_as_absolute(file)
-  },
-  error = function(e) {
-    stop(simpleError(paste(
-      "Cannot create:\n\t", file,
-      "\nBecause the directory does not exist or is not writable."
-    )))
-  })
+      file <- tools::file_path_as_absolute(file)
+    },
+    error = function(e) {
+      stop(simpleError(paste(
+        "Cannot create:\n\t", file,
+        "\nBecause the directory does not exist or is not writable."
+      )))
+    })
+  }
 
   # Determine which TeX engine is being used.
   switch(engine,
@@ -242,7 +245,7 @@ function (file = "./Rplots.tex", width = 7, height = 7,
   packages <- paste( paste( packages, collapse='\n'), collapse='\n')
   footer <- paste( paste( footer,collapse='\n'), collapse='\n')
 
-  .External('tikzDevice', file, width, height, bg, fg, baseSize,
+  .External('TikZ_StartDevice', file, width, height, bg, fg, baseSize,
     standAlone, bareBones, documentDeclaration, packages, footer, console,
     sanitize, engine,
     PACKAGE='tikzDevice')
